@@ -31,12 +31,13 @@ Reusing existing fixtures for a different step name:
 given('I have a beautiful article', fixture='article')
 
 """
-
 import inspect
+import sys
+
 import pytest
 
-from pytest_bdd.types import GIVEN, WHEN, THEN
 from pytest_bdd.feature import remove_prefix
+from pytest_bdd.types import GIVEN, WHEN, THEN
 
 
 class StepError(Exception):
@@ -54,8 +55,7 @@ def given(name, fixture=None):
     """
     name = remove_prefix(name)
     if fixture is not None:
-        frame = inspect.stack()[1]
-        module = inspect.getmodule(frame[0])
+        module = get_caller_module()
         func = getattr(module, fixture, lambda request: request.getfuncargvalue(fixture))
         setattr(module, name, pytest.fixture(lambda: func))
         return _not_a_fixture_decorator
@@ -96,7 +96,6 @@ def _step_decorator(step_type, step_name):
     :param step_name: Step name as in the feature file.
 
     :return: Decorator function for the step.
-    :raises: StepError when step types mismatch.
 
     :note: If the step type is GIVEN it will automatically apply the pytest
     fixture decorator to the step function.
@@ -104,15 +103,21 @@ def _step_decorator(step_type, step_name):
     step_name = remove_prefix(step_name)
 
     def decorator(func):
-        func_ = func
-        frame = inspect.stack()[1]
-        module = inspect.getmodule(frame[0])
+        step_func = func
         if step_type == GIVEN:
-            func = pytest.fixture(func)
-            func_ = lambda request: request.getfuncargvalue(func.func_name)
+            if not hasattr(func, '_pytestfixturefunction'):
+                # avoid overfixturing of a fixture
+                func = pytest.fixture(func)
+            step_func = lambda request: request.getfuncargvalue(func.func_name)
 
-        func_.__name__ = step_name
-        setattr(module, step_name, pytest.fixture(lambda: func_))
+        step_func.__name__ = step_name
+        setattr(get_caller_module(), step_name, pytest.fixture(lambda: step_func))
         return func
 
     return decorator
+
+
+def get_caller_module(depth=2):
+    """Return the module of the caller."""
+    frame = sys._getframe(depth)
+    return inspect.getmodule(frame)
